/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kde_transparency.h"
#include "debug.h"
#include "config_dialog.h"
#include "chat.h"
#include "modules.h"

KDETransparencySlots *kdeTransparencyObj;

extern "C" int kde_transparency_init()
{
	kdebugf();
	kdeTransparencyObj=new KDETransparencySlots();
	return 0;
}
extern "C" void kde_transparency_close()
{
	kdebugf();
	delete kdeTransparencyObj;
	kdeTransparencyObj=NULL;
}

KDETransparencySlots::KDETransparencySlots()
{
	ConfigDialog::registerSlotOnCreate(this, SLOT(onCreateConfigDialog()));
	ConfigDialog::registerSlotOnClose(this, SLOT(onCloseConfigDialog()));
	connect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));
	connect(chat_manager, SIGNAL(chatDestroying(const UinsList&)), this, SLOT(chatDestroying(const UinsList&)));
}

KDETransparencySlots::~KDETransparencySlots()
{
	disconnect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));
	disconnect(chat_manager, SIGNAL(chatDestroying(const UinsList&)), this, SLOT(chatDestroying(const UinsList&)));
	ConfigDialog::unregisterSlotOnCreate(this, SLOT(onCreateConfigDialog()));
	ConfigDialog::unregisterSlotOnClose(this, SLOT(onCloseConfigDialog()));
	for(QMap<QString, KRootPixmap*>::iterator i=rootPixmaps.begin(); i!=rootPixmaps.end(); )
	{
		KRootPixmap *rootxpm=i.data();
		rootxpm->stop();
		delete rootxpm;
		rootPixmaps.remove(i);
	}
	for(QMap<Chat *, KRootPixmap*>::iterator i=chatRootPixmaps.begin(); i!=chatRootPixmaps.end(); )
	{
		KRootPixmap *rootxpm=i.data();
		rootxpm->stop();
		delete rootxpm;
		chatRootPixmaps.remove(i);
	}
}

void KDETransparencySlots::onCreateConfigDialog()
{
	kdebugf();
	KRootPixmap *rootxpm = new KRootPixmap(ConfigDialog::configdialog);
	rootxpm->start();
	rootxpm->repaint(true);
	
	rootPixmaps["config_dialog"]=rootxpm;

	modules_manager->moduleIncUsageCount("kde_transparency");
}

void KDETransparencySlots::onCloseConfigDialog()
{
	kdebugf();
	if (rootPixmaps.contains("config_dialog"))
	{
		KRootPixmap *rootxpm=rootPixmaps["config_dialog"];
		rootxpm->stop();
		delete rootxpm;
		rootPixmaps.remove("config_dialog");
		modules_manager->moduleDecUsageCount("kde_transparency");
	}
	else
		kdebug("this window is not transparent\n");
}

void KDETransparencySlots::chatCreated(const UinsList& senders)
{
	kdebugf();
	Chat *chat=chat_manager->findChatByUins(senders);

	KRootPixmap *rootxpm = new KRootPixmap(chat);
	rootxpm->start();
	rootxpm->repaint(true);

	modules_manager->moduleIncUsageCount("kde_transparency");
	
	chatRootPixmaps[chat]=rootxpm;
}

void KDETransparencySlots::chatDestroying(const UinsList& senders)
{
	kdebugf();
	Chat *chat=chat_manager->findChatByUins(senders);

	if (chatRootPixmaps.contains(chat))
	{
		KRootPixmap *rootxpm=chatRootPixmaps[chat];
		rootxpm->stop();
		delete rootxpm;
		chatRootPixmaps.remove(chat);
		modules_manager->moduleDecUsageCount("kde_transparency");
	}
	else
		kdebug("this window is not transparent\n");
}
